/*
 * COPYRIGHT:        See COPYRIGHT.TXT
 * PROJECT:          Ext2 File System Driver for WinNT/2K/XP
 * FILE:             misc.c
 * PROGRAMMER:       Matt Wu <mattwu@163.com>
 * HOMEPAGE:         http://ext2.yeah.net
 * UPDATE HISTORY: 
 */

/* INCLUDES *****************************************************************/

#include "ext2fs.h"

/* GLOBALS ***************************************************************/

extern PEXT2_GLOBAL Ext2Global;

/* DEFINITIONS *************************************************************/

#ifdef ALLOC_PRAGMA
#pragma alloc_text(PAGE, Ext2Log2)
#pragma alloc_text(PAGE, Ext2SysTime)
#pragma alloc_text(PAGE, Ext2InodeTime)
#pragma alloc_text(PAGE, Ext2OEMToUnicode)
#pragma alloc_text(PAGE, Ext2UnicodeToOEM)
#endif

ULONG
Ext2Log2(ULONG Value)
{
    ULONG Order = 0;

    ASSERT(Value > 0);

    while (Value) {
        Order++;
        Value >>= 1;
    }

    return (Order - 1);
}

LARGE_INTEGER
Ext2SysTime (IN ULONG i_time)
{
    LARGE_INTEGER SysTime;

    RtlSecondsSince1970ToTime(i_time, &SysTime);

    return SysTime;
}

ULONG
Ext2InodeTime (IN LARGE_INTEGER SysTime)
{
    ULONG   Ext2Time;

    RtlTimeToSecondsSince1970(&SysTime, &Ext2Time);

    return Ext2Time;
}


ULONG
Ext2MbsToUnicode(
    IN OUT PUNICODE_STRING Unicode,
    IN     PANSI_STRING    Mbs   )
{
    ULONG Length = 0;
    int i, mbc = 0;
    WCHAR  uc;

    /* Count the length of the resulting Unicode. */
    for (i = 0; i < Mbs->Length; i += mbc) {

        mbc = PAGE_TABLE->char2uni(
                    (PUCHAR)&(Mbs->Buffer[i]),
                    Mbs->Length - i,
                    &uc
                    );

        if (mbc <= 0) {

            /* Invalid character. */
            return 0;
        }

        Length += 2;
    }

    if (Unicode) {
        if (Unicode->MaximumLength < Length) {

            DbgBreak();
            return 0;
        }

        Unicode->Length = 0;
        mbc = 0;

        for (i = 0; i < Mbs->Length; i += mbc) {

            mbc = PAGE_TABLE->char2uni(
                    (PUCHAR)&(Mbs->Buffer[i]),
                    Mbs->Length - i,
                    &uc
                    );
            Unicode->Buffer[Unicode->Length/2] = uc;

            Unicode->Length += 2;
        }
    }

    return Length;
}

ULONG
Ext2UnicodeToMbs (
         IN OUT PANSI_STRING Mbs,
         IN PUNICODE_STRING  Unicode)
{
    ULONG Length = 0;
    UCHAR mbs[0x10];
    int i, mbc;

    /* Count the length of the resulting mbc-8. */
    for (i = 0; i < (Unicode->Length / 2); i++) {

        RtlZeroMemory(mbs, 0x10);
        mbc = PAGE_TABLE->uni2char(
                    Unicode->Buffer[i],
                    mbs,
                    0x10
                    );

        if (mbc <= 0) {

            /* Invalid character. */
            return 0;
        }

        Length += mbc;
    }

    if (Mbs) {

        if (Mbs->MaximumLength < Length) {

            DbgBreak();
            return 0;
        }

        Mbs->Length = 0;

        for (i = 0; i < (Unicode->Length / 2); i++) {

            mbc = PAGE_TABLE->uni2char(
                    Unicode->Buffer[i],
                    mbs,
                    0x10
                    );

            RtlCopyMemory(
                    (PUCHAR)&(Mbs->Buffer[Mbs->Length]),
                    &mbs[0],
                    mbc
                    );

            Mbs->Length += (USHORT)mbc;
        }
    }

    return Length;
}


ULONG
Ext2OEMToUnicodeSize(
        IN PANSI_STRING Oem
        )
{
    if (PAGE_TABLE) {
        return Ext2MbsToUnicode(NULL, Oem);
    }

    return RtlOemStringToCountedUnicodeSize(Oem);
}


NTSTATUS
Ext2OEMToUnicode(
         IN OUT PUNICODE_STRING Unicode,
         IN     POEM_STRING     Oem    )
{
    NTSTATUS  Status;


    if (PAGE_TABLE) {
        Status = Ext2MbsToUnicode(Unicode, Oem);

        if (Status == Unicode->Length) {
            Status = STATUS_SUCCESS;
        } else {
            Status = STATUS_UNSUCCESSFUL;
        }

        goto errorout;
    }


    Status = RtlOemStringToUnicodeString(
                    Unicode, 
                    Oem,
                    FALSE );

    if (!NT_SUCCESS(Status))
    {
        DbgBreak();
        goto errorout;
    }

errorout:

    return Status;
}

ULONG
Ext2UnicodeToOEMSize(
        IN PUNICODE_STRING Unicode
        )
{
    if (PAGE_TABLE) {

        return Ext2UnicodeToMbs(NULL, Unicode);
    }

    return RtlxUnicodeStringToOemSize(Unicode);
}


NTSTATUS
Ext2UnicodeToOEM (
         IN OUT POEM_STRING Oem,
         IN PUNICODE_STRING Unicode)
{
    NTSTATUS Status;

    if (PAGE_TABLE) {

        Status = Ext2UnicodeToMbs(Oem, Unicode);

        if (Status == Oem->Length) {
            Status = STATUS_SUCCESS;
        } else {
            Status = STATUS_UNSUCCESSFUL;
        }

        goto errorout;
    }

    Status = RtlUnicodeStringToOemString(
                            Oem,
                            Unicode,
                            FALSE );

    if (!NT_SUCCESS(Status))
    {
        DbgBreak();
        goto errorout;
    }
    
errorout:

    return Status;
}

